@@ -20,6 +20,7 @@ import uuid
 from logging import DEBUG, WARNING
 
 from typing import cast
+from types import SimpleNamespace as sn
 
 import numpy as np
 import json
@@ -36,7 +37,7 @@ from clu.model import Model
 from basecam import BaseCamera, CameraSystem, Exposure
 #from basecam.actor import CameraActor
 from basecam.events import CameraEvent
-from basecam.mixins import CoolerMixIn, ExposureTypeMixIn, ImageAreaMixIn, ShutterMixIn, CoolerMixIn
+from basecam.mixins import CoolerMixIn, ExposureTypeMixIn, ImageAreaMixIn, ShutterMixIn
 from basecam.notifier import EventListener
 from basecam.utils import cancel_task
 
@@ -103,10 +104,10 @@ class SkymakerCamera(BaseCamera, ExposureTypeMixIn, ImageAreaMixIn, CoolerMixIn)
             k = key.split('.', maxsplit=1)
             c = config.get(k[0] if not k[0].isnumeric() else int(k[0]))  # keys can be numeric
             return d if c is None else c if len(k) < 2 else g(c, k[1], d) if type(c) is dict else d
-        return g(self.config, key, default)
+        return g(self.camera_params, key, default)
 
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, camera_params = None, **kwargs):
 
         super().__init__(*args, **kwargs)
 
@@ -114,36 +115,36 @@ class SkymakerCamera(BaseCamera, ExposureTypeMixIn, ImageAreaMixIn, CoolerMixIn)
         self.logger.sh.formatter = StreamFormatter(fmt='%(asctime)s %(name)s %(levelname)s %(filename)s:%(lineno)d: \033[1m%(message)s\033[21m') 
         self.logger.debug("construct")
 
-        self.config = kwargs['camera_params']
+        self.actor = self.camera_params.get('actor', None)
+        self.scraper_data = self.camera_params.get('scraper_data', {})
+        
         self.inst_params = params_load(f"skymakercam.params.{self.config_get('instpar', None)}")
         self.inst_params.catalog_path = os.path.expandvars(self.config_get('catalog_path', tempfile.TemporaryDirectory()))
         if not os.path.exists(self.inst_params.catalog_path):
             self.log(f"Creating catalog path: {self.inst_params.catalog_path}", WARNING)
             pathlib.Path(self.inst_params.catalog_path).mkdir(parents=True, exist_ok=True)
 
-        rmqname = f"proxy-{uuid.uuid4().hex[:8]}"
+#        rmqname = f"proxy-{uuid.uuid4().hex[:8]}"
 #        self.logger.debug(f"{rmqname} {self.config_get('rmq.host', 'localhost')}")
         
 #        Proxy.setDefaultAmqpc(AMQPClient(name=rmqname, url=os.getenv("RMQ_URL", None)))
 
-        self._tcs = Proxy(self.config_get('tcs', None))
+#        self._tcs = Proxy(self.config_get('tcs', None))
         
-        self.tcs_coord = None
-        self.tcs_pa = 0
-        self.ra_off = 0.0
-        self.dec_off = 0.0
+        self.tcs_coord = SkyCoord(ra=0.0*u.deg, dec=90.0*u.deg)
+        self.tcs_pa = 0.0
 
         self.guide_stars = None
         
         # we do reuse the AMQPClient
-        self._focus_stage = Proxy(self.config_get('focus_stage', None))
+        #self._focus_stage = Proxy(self.config_get('focus_stage', None))
         self.sky_flux = self.config_get('default.sky_flux', 15)
         self.seeing_arcsec = self.config_get('default.seeing_arcsec', 3.5)
 #        self.exp_time = self.config_get('default.exp_time',5)
         
         # we do reuse the AMQPClient
-        self._kmirror = Proxy(self.config_get('kmirror', None))
-        self.kmirror_angle = 0.
+#        self._kmirror = Proxy(self.config_get('kmirror', None))
+#        self.kmirror_angle = 0.
         
         self.site = self.config_get('site', "LCO")
         self.sid = Siderostat()
@@ -166,20 +167,17 @@ class SkymakerCamera(BaseCamera, ExposureTypeMixIn, ImageAreaMixIn, CoolerMixIn)
 
         self.image_namer.dirname = EXPOSURE_DIR.name
 
-    async def tcs_get_position_j2000(self):
-        status = await self._tcs.status()
-        return SkyCoord(ra=status["ra_j2000_hours"]*u.hour, dec=status["dec_j2000_degs"]*u.deg), status["field_angle_here_degs"]
-
+    #def tcs_get_position_j2000(self):
+        #status = await self._tcs.status()
+        
+        #return SkyCoord(ra=status["ra_j2000_hours"]*u.hour, dec=status["dec_j2000_degs"]*u.deg), status["field_angle_here_degs"]
 
     async def _connect_internal(self, **connection_params):
         self.logger.debug(f"connecting ...")
         
-        await self._tcs.start()
-        await self._focus_stage.start()
-        await self._kmirror.start()
-
-        self.tcs_coord = SkyCoord(ra=0.0*u.hour, dec=0.0*u.deg)
-        self.tcs_pa = 0.0
+        #await self._tcs.start()
+        #await self._focus_stage.start()
+        #await self._kmirror.start()
 
         return True
 
@@ -187,24 +185,26 @@ class SkymakerCamera(BaseCamera, ExposureTypeMixIn, ImageAreaMixIn, CoolerMixIn)
         """Close connection to camera.
         """
         self.logger.debug("disconnect")
-        await self._tcs.client.stop()
-        await self._focus_stage.client.stop()
-        await self._kmirror.client.stop()
+        #await self._tcs.client.stop()
+        #await self._focus_stage.client.stop()
+        #await self._kmirror.client.stop()
 
-    async def create_synthetic_image(self, exposure):
-        self.defocus = (math.fabs((await self._focus_stage.getPosition())["Position"] )/100)**2
-        self.log(f"defocus {self.defocus}")
+    async def create_synthetic_image(self, exposure, ra_h=0.0, dec_d=90.0, pa_d=0.0, km_d=0.0, foc_um=0.0, **kwargs):
 
-        self.kmirror_angle = float((await self._kmirror.getPosition())["Position"])
-        self.log(f"kmirror angle (deg): {self.kmirror_angle}")
+        self.log(f"focus um {foc_um}")
+        defocus = math.fabs(foc_um)/100**2
+        self.log(f"defocus {defocus}")
 
-        tcs_coord_current, self.tcs_pa = await self.tcs_get_position_j2000()
-        
-        mathar_angle = math.degrees(self.sid.fieldAngle(self.geoloc, Target(tcs_coord_current), None))
-        self.log(f"mathar angle (deg): {mathar_angle}")
+        self.log(f"kmirror angle (deg): {km_d}")
 
-        sky_angle = self.kmirror_angle - mathar_angle
-        self.log(f"sky angle (deg): {sky_angle}")
+        tcs_coord_current = SkyCoord(ra=ra_h*u.hour, dec=dec_d*u.deg)
+        self.log(f"SkyCoord: {tcs_coord_current}")
+
+        mathar_angle_d = math.degrees(self.sid.fieldAngle(self.geoloc, Target(tcs_coord_current), None))
+        self.log(f"mathar angle (deg): {mathar_angle_d}")
+
+        sky_angle = km_d - mathar_angle_d
+        self.log(f"sky angle (deg): {sky_angle} {pa_d}")
 
         separation = self.tcs_coord.separation(tcs_coord_current)
         self.log(f"separation {separation.arcminute }")
@@ -221,7 +221,7 @@ class SkymakerCamera(BaseCamera, ExposureTypeMixIn, ImageAreaMixIn, CoolerMixIn)
                                     exp_time=exposure.exptime,
                                     seeing_arcsec=self.seeing_arcsec,
                                     sky_flux=self.sky_flux,
-                                    defocus=self.defocus)
+                                    defocus=defocus)
 
     async def _expose_internal(self, exposure, **kwargs):
 
@@ -232,6 +232,8 @@ class SkymakerCamera(BaseCamera, ExposureTypeMixIn, ImageAreaMixIn, CoolerMixIn)
             return arr.reshape(shape).sum(-1).sum(1)
 
 
+#        self.logger.debug(f"kwargs {kwargs}")
+
         image_type = exposure.image_type
 
         #if image_type in ["bias", "dark"]:
@@ -239,7 +241,19 @@ class SkymakerCamera(BaseCamera, ExposureTypeMixIn, ImageAreaMixIn, CoolerMixIn)
         #else:
             #await self.set_shutter(True)
 
-        data = await self.loop.create_task(self.create_synthetic_image(exposure))
+        params = {k: v.val for k,v in self.scraper_data.items()}
+        
+        if kmirror_angle := kwargs.get("km_d", 0.0):
+            params["km_d"] = kmirror_angle
+
+        if ra_h := kwargs.get("ra_h", None):
+            if dec_d := kwargs.get("dec_d", None):
+                params["ra_h"] = ra_h
+                params["dec_d"] = dec_d
+
+#        self.log(f"params {params}")
+
+        data = await self.loop.create_task(self.create_synthetic_image(exposure, **params))
 
         self.notify(CameraEvent.EXPOSURE_READING)
 
@@ -256,12 +270,6 @@ class SkymakerCamera(BaseCamera, ExposureTypeMixIn, ImageAreaMixIn, CoolerMixIn)
         self.notify(CameraEvent.EXPOSURE_POST_PROCESS_DONE)
         return exposure
 
-    #async def _set_shutter_internal(self, shutter_open):
-        #self._shutter_position = shutter_open
-
-    #async def _get_shutter_internal(self):
-        #return self._shutter_position
-
     def _status_internal(self):
         return {"temperature": self._temperature, "cooler": math.nan}
 
